// Copyright 2019-2021 The Inspektor Gadget authors
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Code generated by client-gen. DO NOT EDIT.

package fake

import (
	"context"

	v1alpha1 "github.com/inspektor-gadget/inspektor-gadget/pkg/apis/gadget/v1alpha1"
	v1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	labels "k8s.io/apimachinery/pkg/labels"
	schema "k8s.io/apimachinery/pkg/runtime/schema"
	types "k8s.io/apimachinery/pkg/types"
	watch "k8s.io/apimachinery/pkg/watch"
	testing "k8s.io/client-go/testing"
)

// FakeTraces implements TraceInterface
type FakeTraces struct {
	Fake *FakeGadgetV1alpha1
	ns   string
}

var tracesResource = schema.GroupVersionResource{Group: "gadget", Version: "v1alpha1", Resource: "traces"}

var tracesKind = schema.GroupVersionKind{Group: "gadget", Version: "v1alpha1", Kind: "Trace"}

// Get takes name of the trace, and returns the corresponding trace object, and an error if there is any.
func (c *FakeTraces) Get(ctx context.Context, name string, options v1.GetOptions) (result *v1alpha1.Trace, err error) {
	obj, err := c.Fake.
		Invokes(testing.NewGetAction(tracesResource, c.ns, name), &v1alpha1.Trace{})

	if obj == nil {
		return nil, err
	}
	return obj.(*v1alpha1.Trace), err
}

// List takes label and field selectors, and returns the list of Traces that match those selectors.
func (c *FakeTraces) List(ctx context.Context, opts v1.ListOptions) (result *v1alpha1.TraceList, err error) {
	obj, err := c.Fake.
		Invokes(testing.NewListAction(tracesResource, tracesKind, c.ns, opts), &v1alpha1.TraceList{})

	if obj == nil {
		return nil, err
	}

	label, _, _ := testing.ExtractFromListOptions(opts)
	if label == nil {
		label = labels.Everything()
	}
	list := &v1alpha1.TraceList{ListMeta: obj.(*v1alpha1.TraceList).ListMeta}
	for _, item := range obj.(*v1alpha1.TraceList).Items {
		if label.Matches(labels.Set(item.Labels)) {
			list.Items = append(list.Items, item)
		}
	}
	return list, err
}

// Watch returns a watch.Interface that watches the requested traces.
func (c *FakeTraces) Watch(ctx context.Context, opts v1.ListOptions) (watch.Interface, error) {
	return c.Fake.
		InvokesWatch(testing.NewWatchAction(tracesResource, c.ns, opts))

}

// Create takes the representation of a trace and creates it.  Returns the server's representation of the trace, and an error, if there is any.
func (c *FakeTraces) Create(ctx context.Context, trace *v1alpha1.Trace, opts v1.CreateOptions) (result *v1alpha1.Trace, err error) {
	obj, err := c.Fake.
		Invokes(testing.NewCreateAction(tracesResource, c.ns, trace), &v1alpha1.Trace{})

	if obj == nil {
		return nil, err
	}
	return obj.(*v1alpha1.Trace), err
}

// Update takes the representation of a trace and updates it. Returns the server's representation of the trace, and an error, if there is any.
func (c *FakeTraces) Update(ctx context.Context, trace *v1alpha1.Trace, opts v1.UpdateOptions) (result *v1alpha1.Trace, err error) {
	obj, err := c.Fake.
		Invokes(testing.NewUpdateAction(tracesResource, c.ns, trace), &v1alpha1.Trace{})

	if obj == nil {
		return nil, err
	}
	return obj.(*v1alpha1.Trace), err
}

// UpdateStatus was generated because the type contains a Status member.
// Add a +genclient:noStatus comment above the type to avoid generating UpdateStatus().
func (c *FakeTraces) UpdateStatus(ctx context.Context, trace *v1alpha1.Trace, opts v1.UpdateOptions) (*v1alpha1.Trace, error) {
	obj, err := c.Fake.
		Invokes(testing.NewUpdateSubresourceAction(tracesResource, "status", c.ns, trace), &v1alpha1.Trace{})

	if obj == nil {
		return nil, err
	}
	return obj.(*v1alpha1.Trace), err
}

// Delete takes name of the trace and deletes it. Returns an error if one occurs.
func (c *FakeTraces) Delete(ctx context.Context, name string, opts v1.DeleteOptions) error {
	_, err := c.Fake.
		Invokes(testing.NewDeleteAction(tracesResource, c.ns, name), &v1alpha1.Trace{})

	return err
}

// DeleteCollection deletes a collection of objects.
func (c *FakeTraces) DeleteCollection(ctx context.Context, opts v1.DeleteOptions, listOpts v1.ListOptions) error {
	action := testing.NewDeleteCollectionAction(tracesResource, c.ns, listOpts)

	_, err := c.Fake.Invokes(action, &v1alpha1.TraceList{})
	return err
}

// Patch applies the patch and returns the patched trace.
func (c *FakeTraces) Patch(ctx context.Context, name string, pt types.PatchType, data []byte, opts v1.PatchOptions, subresources ...string) (result *v1alpha1.Trace, err error) {
	obj, err := c.Fake.
		Invokes(testing.NewPatchSubresourceAction(tracesResource, c.ns, name, pt, data, subresources...), &v1alpha1.Trace{})

	if obj == nil {
		return nil, err
	}
	return obj.(*v1alpha1.Trace), err
}
